local addonInfo, internal = ...

---------- init namespace ---------

local intFct     = internal.fct
local uiElements = internal.uiElements
local data       = internal.data

---------- init local variables ---------

local _selectedButton = nil
local _selectedInfo   = nil
local _mainColor      = {r = 0.153, g = 0.314, b = 0.490, a = 1}

local tooltipFoundEventHandler
local tooltipFoundEvent 

---------- init variables ---------

---------- local function block ---------

local function _fctTooltip (_, type, info)

  if _selectedButton == nil then return end
  
  local itemCategory, abilityId, itemType

  if type == "item" or type == "itemtype" then
    local temp = Inspect.Item.Detail(info)
    if temp ~= nil then   
      itemCategory, itemType = temp.category, temp.type
    end
  elseif type == "ability" then
    abilityId = info
  end

  local sectionType = _selectedButton:GetSectionType()
  
  if itemType ~= nil then
    _selectedInfo = { type = "item", info = itemType }
    tooltipFoundEventHandler()
    return
  end

  if itemCategory ~= nil then
    _selectedInfo = { type = "item", info = info }
    tooltipFoundEventHandler()
    return
  end
  
  if abilityId ~= nil then
    _selectedInfo = { type = "ability", info = info }
    tooltipFoundEventHandler()
    return
  end
  
end

local function uiSectionIcon (name, parent, sectionType)

  local canvas = EnKai.uiCreateFrame("nkCanvas", name, parent)
  local texture = EnKai.uiCreateFrame("nkTexture", name .. ".texture", canvas)
  
  local stroke = {r = _mainColor.r, g = _mainColor.g, b = _mainColor.b, a = _mainColor.a, thickness = 1 }
  local path = {  {xProportional = 0, yProportional = 0}, {xProportional = 1, yProportional = 0}, {xProportional = 1, yProportional = 1},
                  {xProportional = 0, yProportional = 1}, {xProportional = 0, yProportional = 0}}
  local fill = {type = 'solid', r = 1, g = 1, b = 1, a = 0}
  
  local selected = false
  
  canvas:SetShape(path, fill, stroke)
  canvas:SetWidth(23)
  canvas:SetHeight(23)
  
  texture:SetPoint("TOPLEFT", canvas, "TOPLEFT", 1, 1)
  texture:SetWidth(22)
  texture:SetHeight(22)
  
  function canvas:destroy()
    EnKai.uiAddToGarbageCollector ('nkCanvas', canvas)
    EnKai.uiAddToGarbageCollector ('nkTexture', texture)
  end
  
  function canvas:GetSectionType() return sectionType end
  
  function canvas:SetSelected(flag)
    selected = flag
  
    _selectedInfo = nil
  
    if flag == true then
      --local thisStroke = {r = 1, g = 0.847, b = 0, a = 1, thickness = 1 }
      local thisStroke = {r = 1, g = 1, b = 1, a = 1, thickness = 1 }
      canvas:SetShape(path, fill, thisStroke)
      _selectedButton = canvas
      
      Command.Event.Attach(tooltipFoundEvent, function ()
        
        if _selectedInfo == nil then return end        
        
        canvas:SetContent(_selectedInfo)
        nkRadialSetup.slotIcon[sectionType] = _selectedInfo 
        
      end, "nkRadial.TooltipFound")
      
    else
      canvas:SetShape(path, fill, stroke)
      _selectedButton = nil
      
      Command.Event.Detach(tooltipFoundEvent, nil, "nkRadial.TooltipFound")
    end
  end
  
  canvas:EventAttach(Event.UI.Input.Mouse.Left.Down, function ()

     local selectedFlag = selected 
     if _selectedButton ~= nil then _selectedButton:SetSelected(false) end
     if selectedFlag == false then canvas:SetSelected(true) end
     
  end, name .. ".Left.Down")
  
  canvas:EventAttach(Event.UI.Input.Mouse.Right.Up, function ()

     nkRadialSetup.slotIcon[sectionType] = data.slotIconDefaults[sectionType]
     canvas:SetContent(data.slotIconDefaults[sectionType])
     
  end, name .. ".Right.Up")
  
  function canvas:SetContent (contentInfo)
    
    local addon, icon = nil, nil
    
    if contentInfo.type == 'item' then    
      local details = Inspect.Item.Detail(contentInfo.info)      
      if details ~= nil then addon, icon = "Rift", details.icon end
      
    elseif contentInfo.type == 'ability' then
      local details = Inspect.Ability.New.Detail(contentInfo.info)
      if details ~= nil then addon, icon = "Rift", details.icon end
      
    elseif contentInfo.type == 'addon' then 
      addon, icon = "nkRadial", contentInfo.path 
    end
    
    if icon ~= nil then      
      texture:SetTextureAsync(addon, icon)
      texture:SetVisible(true)
      
      for idx = 1, data.maxButtons, 1 do
        if uiElements.buttons[idx].section == sectionType then
          uiElements.buttons[idx].main:SetTexture(addon, icon)
        end
      end 
    end
    
  end
  
  return canvas

end

local function uiConfigButton (name, parent, sectionType, count)

  local canvas = EnKai.uiCreateFrame("nkCanvas", name, parent)
  local texture = EnKai.uiCreateFrame("nkTexture", name .. ".texture", canvas)
  
  local stroke = EnKai.art.GetThemeColor('elementMainColor')
  stroke.thickness = 1
  
  local path = {  {xProportional = 0, yProportional = 0}, {xProportional = 1, yProportional = 0}, {xProportional = 1, yProportional = 1},
                  {xProportional = 0, yProportional = 1}, {xProportional = 0, yProportional = 0}}
  local fill = EnKai.art.GetThemeColor('elementSubColor')
  fill.type = 'solid' 
  
  local selected = false
  
  canvas:SetShape(path, fill, stroke)
  canvas:SetWidth(23)
  canvas:SetHeight(23)
  
  texture:SetPoint("TOPLEFT", canvas, "TOPLEFT", 1, 1)
  texture:SetWidth(22)
  texture:SetHeight(22)
  texture:SetVisible(false)
  
  function canvas:destroy()
    EnKai.uiAddToGarbageCollector ('nkCanvas', canvas)
    EnKai.uiAddToGarbageCollector ('nkTexture', texture)
  end
  
  function canvas:GetSectionType() return sectionType end
  
  function canvas:SetSelected(flag)
    selected = flag
  
    _selectedInfo = nil
  
    if flag == true then
      local thisStroke = {r = 1, g = 1, b = 1, a = 1, thickness = 1 }
      canvas:SetShape(path, fill, thisStroke)
      _selectedButton = canvas
      
      Command.Event.Attach(tooltipFoundEvent, function ()
        
        if _selectedInfo == nil then return end        
        
        canvas:SetContent(_selectedInfo)
        nkRadialSetup.slotSetup[sectionType][count] = _selectedInfo 
        
      end, "nkRadial.TooltipFound")
      
    else
      canvas:SetShape(path, fill, stroke)
      _selectedButton = nil
      
      Command.Event.Detach(tooltipFoundEvent, nil, "nkRadial.TooltipFound")
    end
  end
  
  canvas:EventAttach(Event.UI.Input.Mouse.Left.Down, function ()

     local selectedFlag = selected 
     
     if _selectedButton ~= nil then _selectedButton:SetSelected(false) end
     if selectedFlag == false then canvas:SetSelected(true) end
     
  end, name .. ".Left.Down")
  
  canvas:EventAttach(Event.UI.Input.Mouse.Right.Up, function ()

     nkRadialSetup.slotSetup[sectionType][count] = {} 
     texture:SetVisible(false)
     
  end, name .. ".Right.Up")
  
  function canvas:SetContent (contentInfo)
    
    local icon = nil
    
    if contentInfo.type == 'item' then
      local details = Inspect.Item.Detail(contentInfo.info)      
      if details ~= nil then icon = details.icon end
    elseif contentInfo.type == 'ability' then
      local details = Inspect.Ability.New.Detail(contentInfo.info)
      if details ~= nil then icon = details.icon end      
    end
    
    if icon ~= nil then
      texture:SetTextureAsync("Rift", icon)
      texture:SetVisible(true)
    end
    
  end
  
  return canvas

end

local function uiConfigSection (sectionType, name, parent)

  local frame = EnKai.uiCreateFrame("nkFrame", name, parent)
  local checkbox = EnKai.uiCreateFrame("nkCheckbox", name .. ".checkbox", frame)
  local label = EnKai.uiCreateFrame("nkText", name .. ".label", frame)
  local sectionTypeIcon = uiSectionIcon (name .. ".sectionTypeIcon", frame, sectionType)
  
  local buttonFrame = EnKai.uiCreateFrame("nkFrame", name .. ".buttonFrame", frame)
  
  local buttons = {}
  
  frame:SetWidth(250)
  frame:SetHeight(80)
  
  sectionTypeIcon:SetPoint("TOPLEFT", buttonFrame, "TOPLEFT")
  sectionTypeIcon:SetContent(nkRadialSetup.slotIcon[sectionType])
    
  local from, object, to, x, y = "TOPLEFT", sectionTypeIcon, "TOPLEFT", 50, 0
  
  for idx = 1, 10, 1 do
    local thisButton = uiConfigButton (name .. ".button" .. idx, buttonFrame, sectionType, idx)
    thisButton:SetPoint(from, object, to, x, y)
    
    if idx == 5 then    
      to, object, x, y= "BOTTOMLEFT", buttons[1], 0, 2
    else
      to, object, x, y = "TOPRIGHT", thisButton, 5, 0
    end
    
    if #nkRadialSetup.slotSetup[sectionType] >= idx then
      thisButton:SetContent(nkRadialSetup.slotSetup[sectionType][idx])
    end
    
    table.insert(buttons, thisButton)
  end

  checkbox:SetPoint("TOPLEFT", frame, "TOPLEFT")
  checkbox:SetChecked(nkRadialSetup.sections[sectionType])
  checkbox:SetLabelWidth(0)
  checkbox:SetColor(1, 1, 1, 1)
  
  label:SetText(internal.lang.configSection[sectionType])
  label:SetPoint("CENTERLEFT", checkbox, "CENTERRIGHT")
  label:SetFontSize(14)
    
  buttonFrame:SetPoint("TOPLEFT", checkbox, "BOTTOMLEFT", 0, 10)
  
  Command.Event.Attach(EnKai.events[checkbox:GetName()].CheckboxChanged, function (_, flag)
    nkRadialSetup.sections[sectionType] = flag    
    intFct.buildButtons()    
  end, checkbox:GetRealName() .. '.CheckboxChanged')
  
  function frame:destroy()
    EnKai.uiAddToGarbageCollector ('nkFrame', frame)
    EnKai.uiAddToGarbageCollector ('nkCheckbox', checkbox)
    EnKai.uiAddToGarbageCollector ('nkText', label)
    EnKai.uiAddToGarbageCollector ('nkFrame', buttonFrame)
    
    for idx = 1, #buttons, 1 do buttons[idx]:destroy() end
  end
  
  return frame  

end

local function uiConfigTabSettings (name, parent)

  local frame = EnKai.uiCreateFrame("nkFrame", name, parent)
  local lockedCheckbox, slider, alphaSlider, scaleSlider, timedCloseCheckbox, timedCloseSlider, innerDesignSelect, outterDesignSelect
  
  function frame:build()
   
    frame:SetWidth(400)
    frame:SetHeight(50)
    
    lockedCheckbox = EnKai.uiCreateFrame("nkCheckbox", name .. '.lockedCheckbox', frame)  
    lockedCheckbox:SetPoint("TOPLEFT", frame, "TOPLEFT", 10, 10)
    lockedCheckbox:SetLabelWidth(100)
    lockedCheckbox:SetText(internal.lang.lockedCheckbox)
    lockedCheckbox:SetChecked(nkRadialSetup.locked)
    
    Command.Event.Attach(EnKai.events[name .. '.lockedCheckbox'].CheckboxChanged, function (_, newValue)   
      nkRadialSetup.locked = newValue
    end, name .. '.lockedCheckbox.CheckboxChanged')
     
    slider = EnKai.uiCreateFrame("nkSlider", name .. ".slider", frame)
    slider:SetRange(1, 360)
    slider:SetMidValue(180)
    slider:SetText(internal.lang.angleSlider) 
    slider:SetPoint("TOPLEFT", lockedCheckbox, "TOPLEFT", 0, 20)
    slider:AdjustValue(nkRadialSetup.startAngle)
            
    Command.Event.Attach(EnKai.events[slider:GetName()].SliderChanged, function (_, newValue)
      nkRadialSetup.startAngle = newValue
      intFct.changeAngle()
    end, slider:GetName() .. '.SliderChanged')
    
    alphaSlider = EnKai.uiCreateFrame("nkSlider", name .. ".alphaSlider", frame)
    alphaSlider:SetRange(20, 100)
    alphaSlider:SetMidValue(50)
    alphaSlider:SetText(internal.lang.alphaSlider) 
    alphaSlider:SetPoint("TOPLEFT", slider, "TOPLEFT", 250, 0)
    alphaSlider:AdjustValue(nkRadialSetup.mouseOverAlpha * 100)
        
    Command.Event.Attach(EnKai.events[alphaSlider:GetName()].SliderChanged, function (_, newValue)
      nkRadialSetup.mouseOverAlpha = newValue / 100
      uiElements.center:SetAlpha(nkRadialSetup.mouseOverAlpha)
    end, alphaSlider:GetName() .. '.SliderChanged')
    
    scaleSlider = EnKai.uiCreateFrame("nkSlider", name .. ".scaleSlider", frame)
    scaleSlider:SetRange(20, 200)
    scaleSlider:SetMidValue(50)
    scaleSlider:SetText(internal.lang.scaleSlider) 
    scaleSlider:SetPoint("TOPLEFT", slider, "BOTTOMLEFT", 0, 10)
    scaleSlider:AdjustValue(nkRadialSetup.scale * 100)
        
    Command.Event.Attach(EnKai.events[scaleSlider:GetName()].SliderChanged, function (_, newValue)
      nkRadialSetup.scale = newValue / 100
      intFct.changeScale()
    end, scaleSlider:GetName() .. '.SliderChanged')
    
    timedCloseCheckbox  = EnKai.uiCreateFrame("nkCheckbox", name .. ".timedCloseCheckbox", frame)
    timedCloseCheckbox:SetChecked(nkRadialSetup.timedClose)
    timedCloseCheckbox:SetLabelWidth(100)
    timedCloseCheckbox:SetText(internal.lang.timedCloseCheckbox)
    timedCloseCheckbox:SetPoint("TOPLEFT", scaleSlider, "BOTTOMLEFT",0 ,10)
    
    Command.Event.Attach(EnKai.events[timedCloseCheckbox:GetName()].CheckboxChanged, function (_, newValue)
      nkRadialSetup.timedClose = newValue
    end, timedCloseCheckbox:GetName() .. '.CheckboxChanged')
    
    timedCloseSlider = EnKai.uiCreateFrame("nkSlider", name .. ".timedCloseSlider", frame)
    timedCloseSlider:SetRange(20, 120)
    timedCloseSlider:SetMidValue(60)
    timedCloseSlider:SetText(string.format(internal.lang.timedCloseSlider, nkRadialSetup.closingTime)) 
    timedCloseSlider:SetPoint("TOPLEFT", timedCloseCheckbox, "TOPLEFT", 250, 0)
    timedCloseSlider:AdjustValue(nkRadialSetup.closingTime)
        
    Command.Event.Attach(EnKai.events[timedCloseSlider:GetName()].SliderChanged, function (_, newValue)
      nkRadialSetup.closingTime = newValue      
      timedCloseSlider:SetText(string.format(internal.lang.timedCloseSlider, nkRadialSetup.closingTime))
    end, timedCloseSlider:GetName() .. '.SliderChanged')
    
    innerDesignSelect = EnKai.uiCreateFrame("nkCombobox", name .. ".innerDesignSelect", frame)
    innerDesignSelect:SetPoint("TOPLEFT", timedCloseCheckbox, "BOTTOMLEFT", 0, 30)
    innerDesignSelect:SetWidth(200)
    innerDesignSelect:SetLabelWidth(100)
    innerDesignSelect:SetSelection(internal.lang.designSelect)
    innerDesignSelect:SetSelectedValue(nkRadialSetup.innerDesign, false)
    innerDesignSelect:SetText(internal.lang.innerDesign)
    
    Command.Event.Attach(EnKai.events[name .. ".innerDesignSelect"].ComboChanged, function (_, newValue)
      nkRadialSetup.innerDesign = newValue.value
      if uiElements.center ~= nil then uiElements.center:SetDesign(newValue.value) end            
    end, name .. "innerDesignSelect..ComboChanged")
    
    outterDesignSelect = EnKai.uiCreateFrame("nkCombobox", name .. ".outterDesignSelect", frame)
    outterDesignSelect:SetPoint("TOPLEFT", innerDesignSelect, "TOPLEFT", 250, 0)
    outterDesignSelect:SetWidth(200)
    outterDesignSelect:SetLabelWidth(100)
    outterDesignSelect:SetSelection(internal.lang.designSelect)
    outterDesignSelect:SetSelectedValue(nkRadialSetup.outterDesign, false)
    outterDesignSelect:SetText(internal.lang.outterDesign)
    
    Command.Event.Attach(EnKai.events[name .. ".outterDesignSelect"].ComboChanged, function (_, newValue)
      nkRadialSetup.outterDesign = newValue.value
      
      for k, v in pairs (uiElements.buttons) do
        v.main:SetDesign(newValue.value)
        for idx = 1, #v.sub, 1 do v.sub[idx]:SetDesign(newValue.value) end
      end
      
    end, name .. "outterDesignSelect..ComboChanged")
    
  end

  function frame:destroy()
    slider:destroy()
    alphaSlider:destroy()
    EnKai.uiAddToGarbageCollector ('nkFrame', frame)    
  end  
  
  return frame

end

local function uiConfigTabCategories (name, parent)

  local frame = EnKai.uiCreateFrame("nkFrame", name, parent)
  
  local categories = {}

  function frame:build()
  
    local height = 0  
    local from, object, to, x, y = "TOPLEFT", frame, "TOPLEFT", 10, 10
    
    local even = false
    
    for k, v in pairs (nkRadialSetup.sections) do
      local thisCategory = uiConfigSection(k, name .. ".section." .. k, frame)    
      thisCategory:SetPoint(from, object, to, x, y)
      
      if even then  
        from, to, x, y, even = "TOPLEFT", "BOTTOMLEFT", 0, 10, false
      else
        from, object, to, x, y, even = "TOPLEFT", thisCategory, "TOPRIGHT", 0, 0, true
      end
      
      categories[k] = thisCategory
    end
  end
    
  function frame:destroy()
     EnKai.uiAddToGarbageCollector ('nkFrame', frame)
  end
  
  return frame

end

local function uiConfig ()

  local name = "nkRadial.config"

  local config = EnKai.uiCreateFrame("nkWindowMetro", name, uiElements.context)

  config:SetPoint("CENTER", UIParent, "CENTER")
  config:SetWidth(530)
  config:SetHeight(600)
  config:SetTitle(addonInfo.toc.Identifier .. " ".. addonInfo.toc.Version)
  config:SetCloseable(true)
  config:SetColor({ r = 0.126, g = 0.161, b = 0.196, a = .8, thickness=1}, {type = "solid", r = 0.126, g = 0.161, b = 0.196, a = .8})
  config:SetTitleFontColor(1, 1, 1, 1)
  
  local tabPane = EnKai.uiCreateFrame("nkTabPaneMetro", name .. ".tabPane", config:GetContent())
  tabPane:SetColor({ thickness = 1, r = 0.078, g = 0.188, b = 0.306, a = 1}, { type = 'solid', r = 0.051, g = 0.118, b = 0.192, a = 1}, nil, { r = 1, g = 1, b = 1, a = 1})
  tabPane:SetBorder(false)
  
  local paneTabSettings = uiConfigTabSettings(name .. ".tabSettings", tabPane)
  local paneTabCategories = uiConfigTabCategories(name .. ".tabCategories", tabPane)
  
  local EnKaiLogo = EnKai.uiCreateFrame("nkTexture", name .. ".EnKaiLogo", config)
  EnKaiLogo:SetTextureAsync(EnKai.art.GetThemeLogo()[1],EnKai.art.GetThemeLogo()[2])
  EnKaiLogo:SetPoint("BOTTOMLEFT", config:GetContent(), "BOTTOMLEFT", 10, -5)
  EnKaiLogo:SetWidth(125)
  EnKaiLogo:SetHeight(33)
  
  local versionText = UI.CreateFrame("Text", name .. ".versionText", config)
  versionText:SetFontSize(11)
  versionText:SetText(string.format(internal.lang.txtVersion, addonInfo.toc.Version))
  versionText:SetFontColor(236, 228, 189, 1)
  versionText:SetPoint("BOTTOMRIGHT", tabPane, "BOTTOMRIGHT", -5, -5)
  versionText:SetLayer(99)
  
  local closeButton = EnKai.uiCreateFrame("nkButtonMetro", name .. ".closeButton", config:GetContent())
  
  closeButton:SetPoint("BOTTOMRIGHT", config:GetContent(), "BOTTOMRIGHT", -10, -10)
  closeButton:SetText(internal.lang.close)
  closeButton:SetFontColor(1, 1, 1)
  closeButton:SetColor(_mainColor.r, _mainColor.g, _mainColor.b)
  closeButton:SetIcon("nkRadial", "gfx/iconClose.png")
  closeButton:SetScale(.8)
  closeButton:SetLayer(9)
  
  Command.Event.Attach(EnKai.events[name .. ".closeButton"].Clicked, function (_, newValue)
    uiElements.config:SetVisible(false)   
  end, name .. ".closeButton.Clicked")
  
  local tutorialButton = EnKai.uiCreateFrame("nkButtonMetro", name .. ".tutorialButton", config:GetContent())
  
  tutorialButton:SetPoint("CENTERRIGHT", closeButton, "CENTERLEFT", -10, 0)
  tutorialButton:SetText(internal.lang.tutorial)
  tutorialButton:SetFontColor(1, 1, 1)
  tutorialButton:SetColor(_mainColor.r, _mainColor.g, _mainColor.b)
  tutorialButton:SetIcon("nkRadial", "gfx/iconTutorial.png")
  tutorialButton:SetScale(.8)
  tutorialButton:SetLayer(9)
  
  Command.Event.Attach(EnKai.events[name .. ".tutorialButton"].Clicked, function (_, newValue)
    
    if uiElements.tutorial == nil then  uiElements.tutorial = intFct.tutorial() end
    uiElements.tutorial:SetVisible(true)
    
  end, name .. ".tutorialButton.Clicked")
  
  
  local oSetVisible = config.SetVisible
  
  function config:SetVisible(flag)
  
    if flag == true then
      Command.Event.Attach(Event.Tooltip, _fctTooltip, "nkRadial.Event.Tooltip")      
    else
      Command.Event.Detach(Event.Tooltip, nil, "nkRadial.Event.Tooltip")
    end 
    
    oSetVisible(self, flag)
  
  end
    
  tabPane:SetPoint("TOPLEFT", config:GetContent(), "TOPLEFT", 10, 10)
  tabPane:SetPoint("BOTTOMRIGHT", config:GetContent(), "BOTTOMRIGHT", -10, -50)
  tabPane:SetLayer(1)
   
  tabPane:AddPane( { label = internal.lang.tabHeaderCategories, frame = paneTabCategories, initFunc = function() paneTabCategories:build() end}, false)
  tabPane:AddPane( { label = internal.lang.tabHeaderSettings, frame = paneTabSettings, initFunc = function() paneTabSettings:build() end}, true)
  
  --if EnKai.internal.checkEvents ("nkRadial", true) == false then return nil end
  
  tooltipFoundEventHandler, tooltipFoundEvent = Utility.Event.Create(addonInfo.identifier, "nkRadial.TooltipFound")
  
  config:SetVisible(true)
  
  return config
  
end

---------- addon internal function block ---------

function intFct.ShowConfig()

  if uiElements.config == nil then
    uiElements.config = uiConfig()
  else
    uiElements.config:SetVisible(true)
  end
  
end